﻿// STEP 1 - Add reference - NEXTWUSB_dotNET_12B.dll 
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
//SETP 2 - Add Using NEXTWUSB_dotNET_12B
using NEXTWPCIe_dotNET;

namespace NEXTW_PCIeSample_CSharp
{
    class Program
    {
        static bool checkResponse(uint slaves, uint column, uint expectValue, uint retry)
        {
            uint realValue = 0xFEDCBA98; //inital data
            uint retried;
            for (retried = 0; retried <= retry; retried++)
            {
                NEXTWPCIe.ECMPCIeRead();

                if (column == 0)
                    if (slaves == 41)
                        realValue = NEXTWPCIe.respData[slaves].CMD;
                    else
                        realValue = (uint)(NEXTWPCIe.respData[slaves].CMD & 0xFFFF);
                else if (column == 1)
                {
                    if (slaves == 0)
                        realValue = (uint)(NEXTWPCIe.respData[slaves].Parm & 0xFF);
                    else
                        realValue = NEXTWPCIe.respData[slaves].Parm;
                }
                else if (column == 2)
                    realValue = NEXTWPCIe.respData[slaves].Data1;
                else
                    realValue = NEXTWPCIe.respData[slaves].Data2;

                if (realValue == expectValue)
                {
                    // Console.WriteLine("PASS");
                    break;
                }
                else if (retry == retried)
                {
                    Console.WriteLine("Fail,realValue= " + realValue.ToString() + " expectValue= " + expectValue.ToString() + " retry = " + retried.ToString());
                }
                else
                {
                    NEXTWPCIe.ClearCmdData();
                    NEXTWPCIe.ECMPCIeWriteRead();
                }
            }
            return (realValue == expectValue);
        }
        static void Main(string[] args)
        {
            int retValue;
            bool isOpen = false;
            #region STEP 3 - Open the PCIe Port
            retValue = NEXTWPCIe.OpenECMPCIe();
            if (retValue == 0)
                isOpen = true;
            else
            {
                Console.WriteLine("Open PCIe Fail");
                return;
            }
            System.Threading.Thread.Sleep(100);
            #endregion
            #region STEP 4 - SET_STATE to PreOP
            NEXTWPCIe.ClearCmdData();
            NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_STATE;
            NEXTWPCIe.cmdData[0].Data1 = NEXTWPCIe.STATE_PRE_OP; // set state to pre-OP
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 2, 100);
            #endregion
            #region STEP 5 - SET_AXIS
            NEXTWPCIe.ClearCmdData();
            char[] slaveType = { 'p', 'i', 'd', 's', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n',
                                 'n', 'n', 'n', 'n', 'n', 'n', 'n', 'n'};
            uint topology;
            int j = 0;
            for (int i = 1; i <= 8; i++)
            {
                topology = 0;
                for (; j < i * 8; j++)
                {
                    switch (slaveType[j])
                    {
                        case 'd':  // 'd' means drive		
                            break;
                        case 'i': // 'i' means io
                            topology += (uint)(Math.Pow(16, (j % 8)));
                            break;
                        case 'p': // 'p' means HSP(High Speed Pulse)
                            topology += (uint)(Math.Pow(16, (j % 8)) * 2);
                            break;
                        case 's': // 's' means Step
                            topology += (uint)(Math.Pow(16, (j % 8)) * 3);
                            break;
                        default:
                            break;
                    }
                }
                NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_AXIS;
                NEXTWPCIe.cmdData[0].Parm = (ushort)(i - 1); // Group

                NEXTWPCIe.cmdData[0].Data1 = topology;
                NEXTWPCIe.cmdData[0].Data2 = 0;
                if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                    Console.WriteLine("Write Error");
                System.Threading.Thread.Sleep(2);
            }
            #endregion
            #region STEP 6 - SET_DC
            NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_DC;
            NEXTWPCIe.cmdData[0].Parm = 0;
            NEXTWPCIe.cmdData[0].Data1 = 2000; // set cycle time to 2000 us
            NEXTWPCIe.cmdData[0].Data2 = 0xFFFF; // Auto offet adjustment

            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() !=0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(2);
            #endregion
            #region STEP 7 - Set DRIVE_MODE
            NEXTWPCIe.ClearCmdData();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's')
                {
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.DRIVE_MODE;
                    NEXTWPCIe.cmdData[i].Data1 = NEXTWPCIe.CSP_MODE; // set to CSP mode
                    NEXTWPCIe.cmdData[i].Data2 = NEXTWPCIe.DCSYNC; // set using DC sync
                }
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(2);
            #endregion
            #region STEP 8 - SET_STATE to SafeOP (This step could be skipped)
            NEXTWPCIe.ClearCmdData();
            NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_STATE;
            NEXTWPCIe.cmdData[0].Data1 = NEXTWPCIe.STATE_SAFE_OP;// set state to Safe-OP
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 4, 1000);  // wait for enter to safe op mode
            #endregion
            #region STEP 9 - SET_STATE to OP
            NEXTWPCIe.ClearCmdData();
            NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_STATE;
            NEXTWPCIe.cmdData[0].Data1 = NEXTWPCIe.STATE_OPERATIONAL;// set state to OP
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 8, 1000);// wait for enter to op mode
            #endregion
            #region STEP 10 - SET Servo On
            NEXTWPCIe.ClearCmdData();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's')  //set drive or HSP slave servo on
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.SV_ON;
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(400); // wait for servo on 
            #endregion

            #region STEP 11- Read Current Position
            NEXTWPCIe.ECMPCIeRead();
            int[] TargerPosition = new int[NEXTWPCIe.DEF_MA_MAX];
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX; i++)
            {
                TargerPosition[i] = (int)NEXTWPCIe.respData[i].Data1;
            }
            #endregion

            #region STEP 12- SET Cycle Synchronized Position
            System.Threading.Thread.Sleep(1000); // wait until motion stop totally
            NEXTWPCIe.ClearCmdData();
            NEXTWPCIe.ECMPCIeRead();
            Console.WriteLine("Current Position: Slave 1 = " + NEXTWPCIe.respData[1].Data1.ToString() +
                " Slave 2 = " + NEXTWPCIe.respData[2].Data1.ToString() + ", Slave 3 = " + NEXTWPCIe.respData[3].Data1.ToString());
            for (int i = 0; i < 1000; i++) // send 1000 times, let motor move 1000 * 100 pulse
            {
                for (j = 1; j < NEXTWPCIe.DEF_MA_MAX - 1; j++)
                {
                    if (slaveType[j - 1] == 'd' || slaveType[j - 1] == 'p' || slaveType[j - 1] == 's') //Set drive or HSP slave Cycle Synchronized Position
                    {
                        TargerPosition[j] += 3; // set to move 100 pulse in one cycle time
                        NEXTWPCIe.cmdData[j].CMD = NEXTWPCIe.CSP;
                        NEXTWPCIe.cmdData[j].Data1 = (uint)TargerPosition[j];
                    }
                }
                if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                    Console.WriteLine("Write Error");
                NEXTWPCIe.ECMPCIeRead();
                if ((NEXTWPCIe.respData[0].Data2 % 65536) < 20) // If FIFO remaining < 20 then sleep 10ms
                    System.Threading.Thread.Sleep(10);
            }
            System.Threading.Thread.Sleep(3000); // wait until motion stop totally
            NEXTWPCIe.ECMPCIeRead();
            Console.WriteLine("Current Position: Slave 1 = " + NEXTWPCIe.respData[1].Data1.ToString() +
                " Slave 2 = " + NEXTWPCIe.respData[2].Data1.ToString() + ", Slave 3 = " + NEXTWPCIe.respData[3].Data1.ToString());
            #endregion
            #region STEP 13- SET Servo Off
            NEXTWPCIe.ClearCmdData();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'd' || slaveType[i - 1] == 'p' || slaveType[i - 1] == 's') //Set drive or HSP slave servo off
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.SV_OFF;
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(100);
            #endregion

            #region STEP 14-Read Input
            NEXTWPCIe.ClearCmdData();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //only IO slave can read input
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.IO_RD;
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(10);
            NEXTWPCIe.ECMPCIeRead();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can read input
                    Console.WriteLine("Slave " + i.ToString() + NEXTWPCIe.respData[i].Data1.ToString());
                if (slaveType[i - 1] == 'p') //HSP has 8 channel Inputs
                    Console.WriteLine("Slave " + i.ToString() + (NEXTWPCIe.respData[i].Data2 & 0xFF).ToString());
            }
            System.Threading.Thread.Sleep(100);
            #endregion

            #region STEP 15-Write Output
            NEXTWPCIe.ClearCmdData();
            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can write output
                {
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.IO_WR;
                    NEXTWPCIe.cmdData[i].Data1 = 0xFFFF; //Output Value
                }
                if (slaveType[i - 1] == 'p')
                {
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.CSP;
                    NEXTWPCIe.cmdData[i].Data2 |= 0xFF; //Output Value
                }
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(3000);

            for (int i = 1; i < NEXTWPCIe.DEF_MA_MAX - 1; i++)
            {
                if (slaveType[i - 1] == 'i') //IO slave can write output
                {
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.IO_WR;
                    NEXTWPCIe.cmdData[i].Data1 = 0xFFFF;//Output Value
                }
                if (slaveType[i - 1] == 'p')
                {
                    NEXTWPCIe.cmdData[i].CMD = NEXTWPCIe.CSP;
                    NEXTWPCIe.cmdData[i].Data2 &= 0xFF00; //Output Value
                }
            }
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            System.Threading.Thread.Sleep(1000);
            #endregion

            #region SETP 16- SET_STATE to Init
            NEXTWPCIe.ClearCmdData();
            NEXTWPCIe.cmdData[0].CMD = NEXTWPCIe.SET_STATE;
            NEXTWPCIe.cmdData[0].Data1 = NEXTWPCIe.STATE_INIT;
            if (!isOpen || NEXTWPCIe.ECMPCIeWriteRead() != 0)
                Console.WriteLine("Write Error");
            checkResponse(0, 1, 1, 1000);
            #endregion
            #region STEP 17- Close PCIe
            NEXTWPCIe.CloseECMPCIe();
            #endregion
        }
    }
}
